extends RigidBody2D

const Explosion = preload("res://Explosion.tscn")
const Laser = preload("res://Laser.tscn")

var tengo_control = false
var propulsor = 100
var puedo_disparar = true

var estado_historico = []

signal cambiar_estado(nuevo_estado)
signal aterrizaje
signal abordar
signal destruido
signal actualizar_propulsor(cant)
signal armamento
signal propulsor

enum estado {
	CAPTURADO,
	DESCENDIENDO,
	ATERRIZADO,
	ASCENDIENDO
}

func _ready():
	set_max_contacts_reported(1)
	var _sce = self.connect("cambiar_estado", self, "cambiar_estado")
	var _abo = self.connect("abordar", get_parent(), "lander_abordar")
	var _atr = self.connect("aterrizaje", get_parent(), "lander_aterrizado")
	var _lde = self.connect("destruido", get_parent(), "lander_destruido")
	var _apr = self.connect("actualizar_propulsor", get_parent().get_node("HUD"), "propulsor")
	var _aar = self.connect("armamento", get_parent().get_node("HUD"), "activar_armamento")
	var _aro = self.connect("propulsor", get_parent().get_node("HUD"), "activar_propulsor")
	emit_signal("cambiar_estado", estado.CAPTURADO)
	emit_signal("actualizar_propulsor", propulsor)
	emit_signal("propulsor")

func cambiar_estado(estado):
	estado_historico.append(estado)

func estado_actual():
	return estado_historico.back()

func _integrate_forces(_state):
	if tengo_control:
		if Input.is_action_just_pressed("ui_right"):
			apply_central_impulse(Vector2(10, 0))
		if Input.is_action_just_pressed("ui_left"):
			apply_central_impulse(Vector2(-10, 0))
		#if Input.is_action_just_pressed("ui_down"):
		#	apply_central_impulse(Vector2(0, 20))
		if propulsor > 0 and Input.is_action_just_pressed("ui_up"):
			apply_central_impulse(Vector2(0, -20))
			propulsor -= 5
			emit_signal("actualizar_propulsor", propulsor)
		if estado_actual() == estado.ASCENDIENDO and Input.is_action_just_pressed("ui_up"):
			disparar()

func _physics_process(_delta):
	if get_colliding_bodies().size() > 0:
		if get_colliding_bodies()[0].get_collision_layer_bit(0) or get_colliding_bodies()[0].get_collision_layer_bit(3) or get_colliding_bodies()[0].get_collision_layer_bit(4):
			crear_explosion()
		match estado_actual():
			estado.DESCENDIENDO:
				if get_colliding_bodies()[0].get_collision_layer_bit(2):
					emit_signal("cambiar_estado", estado.ATERRIZADO)
					lander_puede_maniobar(false)
					if $Abordar.is_stopped():
						emit_signal("aterrizaje")
						$Abordar.start()

func disparar():
	if puedo_disparar:
		var b = Laser.instance()
		get_parent().call_deferred("add_child", b)
		b.transform = $Disparo.global_transform
		b.rotation_degrees = 180
		puedo_disparar = false
	if $Disparar.is_stopped():
		$Disparar.start()

func crear_explosion():
	var b = Explosion.instance()
	get_parent().add_child(b)
	b.get_node("Sprite").transform = global_transform
	emit_signal("destruido")
	queue_free()

func _on_Abordar_timeout():
	emit_signal("abordar")

func remover_lander():
	lander_puede_maniobar(false)
	queue_free()

func lander_puede_maniobar(valor):
	tengo_control = valor

func lander_puede_maniobrar():
	lander_puede_maniobar(true)
	emit_signal("cambiar_estado", estado.DESCENDIENDO)

func lander_ascendiendo():
	propulsor = 0
	emit_signal("actualizar_propulsor", propulsor)
	lander_puede_maniobar(true)
	emit_signal("cambiar_estado", estado.ASCENDIENDO)
	emit_signal("armamento")

func _on_Disparar_timeout():
	puedo_disparar = true
